<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 10/30/14, 1:22 PM
 * @copyright 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/
 */

namespace OCA\Firewall\AppInfo;

require_once __DIR__ . '/../3rdparty/autoload.php';

use OCA\Firewall\Config;
use OCA\Firewall\Context;
use OCA\Firewall\Firewall;
use OCA\Firewall\Logger;
use OCA\Firewall\RuleFactory;
use OCA\Firewall\Ruler;
use OCA\Firewall\Rules;
use OCP\AppFramework\App;
use OCP\App as PApp;
use OCP\IContainer;

class Application extends App {

	/**
	 * @param array $urlParams
	 */
	public function __construct(array $urlParams = array()) {
		parent::__construct('firewall', $urlParams);

		$this->registerServices();
	}

	/**
	 * Register any services into the DI Container
	 */
	protected function registerServices() {
		$this->getContainer()->registerService('OCA\Firewall\Config', function (IContainer $c) {
			/** @var \OCP\IServerContainer $server */
			$server = $c->query('ServerContainer');

			return new Config(
				$server->getConfig()
			);
		});

		$this->getContainer()->registerService('OCA\Firewall\Context', function (IContainer $c) {
			/** @var \OCP\IServerContainer $server */
			$server = $c->query('ServerContainer');

			return new Context(
				$server->getRequest(),
				$server->getGroupManager(),
				$server->getUserSession(),
				$server->getMimeTypeDetector(),
				$c->query('OCA\Firewall\Config')
			);
		});

		$this->getContainer()->registerService('OCA\Firewall\RuleFactory', function (IContainer $c) {
			return new RuleFactory($c);
		});

		$this->getContainer()->registerService('OCA\Firewall\Ruler', function () {
			return new Ruler();
		});

		$this->getContainer()->registerService('OCA\Firewall\Rules', function (IContainer $c) {
			return new Rules(
				$c->query('OCA\Firewall\RuleFactory'),
				$c->query('OCA\Firewall\Context')
			);
		});

		$this->getContainer()->registerService('OCA\Firewall\Logger', function (IContainer $c) {
			return new Logger(
				$c->query('ServerContainer')->getLogger(),
				$c->query('OCA\Firewall\Config')->getDebugLevel()
			);
		});

		$this->getContainer()->registerService('OCA\Firewall\Firewall', function (IContainer $c) {
			/** @var \OCP\IServerContainer $server */
			$server = $c->query('ServerContainer');

			return new Firewall(
				$c->query('OCA\Firewall\Config'),
				$c->query('OCA\Firewall\Context'),
				$c->query('OCA\Firewall\Rules'),
				$c->query('OCA\Firewall\Logger'),
				$server->getUserSession(),
				$server->getEventDispatcher(),
				$server->getGroupManager(),
				$server->getRequest(),
				$server->getL10NFactory(),
				$server->getURLGenerator()
			);
		});
	}

	/**
	 * Boot the firewall application
	 */
	public function boot() {
		$this->registerSettings();

		$container = $this->getContainer();
		/** @var Firewall $firewall */
		$firewall = $container->query('OCA\Firewall\Firewall');
		$firewall->boot();
	}

	/**
	 * Register Any needed settings pages
	 */
	public function registerSettings() {
		// register settings page
		PApp::registerAdmin('firewall', 'settings/settings');
	}
}
