<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall;

use OCP\ILogger;

class Logger {
	const LOGGING_OFF = 1;
	const LOGGING_FAILURES = 2;
	const LOGGING_ON = 3;

	/**
	 * @var int
	 */
	protected $logLevel;

	/**
	 * @var ILogger
	 */
	private $log;

	/**
	 * @param ILogger $log
	 * @param int $logLevel
	 */
	public function __construct(ILogger $log, $logLevel) {
		$this->log = $log;
		$this->logLevel = $logLevel;
	}

	/**
	 * Log when the firewall was skipped
	 *
	 * @param bool $isAdmin
	 */
	public function onSkip($isAdmin) {
		if ($this->logLevel === self::LOGGING_ON && $isAdmin) {
			$this->log->debug('Firewall rule skipped for admin', ['app' => 'firewall']);
		} else if ($this->logLevel === self::LOGGING_ON) {
			$this->log->debug('Firewall rule skipped for not logged in user', ['app' => 'firewall']);
		}
	}

	/**
	 * Log the result when the firewall passes
	 */
	public function onPass() {
		if ($this->logLevel === self::LOGGING_ON) {
			$this->log->debug('Firewall rule passed', ['app' => 'firewall']);
		}
	}

	/**
	 * Log the result when the firewall fails
	 *
	 * @param string $offendingRule
	 */
	public function onFail($offendingRule) {
		if ($this->logLevel > self::LOGGING_OFF) {
			$this->log->debug('Firewall validation on rule "{offender}"', [
				'app' => 'firewall',
				'offender' => $offendingRule,
			]);
		}
	}
}
