<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Appinfo;


use OCA\Firewall\Tests\TestCase;

class AppTest extends TestCase {

	/** @var \OCP\App\IAppManager|\PHPUnit_Framework_MockObject_MockObject */
	protected $appManager;

	/** @var \OCP\ILogger|\PHPUnit_Framework_MockObject_MockObject */
	protected $logger;

	/** @var \OC\AppFramework\Http\Request|\PHPUnit_Framework_MockObject_MockObject */
	protected $request;

	protected function setUp() {
		parent::setUp();

		$this->appManager = $this->getMockBuilder('OCP\App\IAppManager')
			->disableOriginalConstructor()
			->getMock();

		$this->logger = $this->getMockBuilder('OCP\ILogger')
			->disableOriginalConstructor()
			->getMock();

		$this->overwriteService('AppManager', $this->appManager);
		$this->overwriteService('Logger', $this->logger);
	}

	protected function tearDown() {
		$this->restoreService('AppManager');
		$this->restoreService('Logger');
		$this->restoreService('Request');

		parent::tearDown();
	}

	public function dataApp() {
		return [
			[true, true, ['server' => ['TRAVIS' => true]]],
			[true, true, []],
			[true, false, ['server' => ['TRAVIS' => true]]],
			[true, false, []],
			[false, false, ['server' => ['TRAVIS' => true]]],
			[false, false, []],
			[false, true, ['server' => ['TRAVIS' => true]]],
			[false, true, []],
		];
	}

	/**
	 * @dataProvider dataApp
	 * @param bool $enterpriseKeyEnabled
	 * @param bool $isCli
	 * @param array $requestVars
	 */
	public function testApp($enterpriseKeyEnabled, $isCli, array $requestVars) {
		$this->request = $this->getMockBuilder('OC\AppFramework\Http\Request')
			->setConstructorArgs([
				$requestVars,
				null,
				$this->getMock('OCP\IConfig'),
				'php://input',
			])
			->getMock();
		$this->overwriteService('Request', $this->request);

		$this->appManager->expects($this->once())
			->method('isEnabledForUser')
			->with('enterprise_key')
			->willReturn($enterpriseKeyEnabled);

		if (!$enterpriseKeyEnabled) {
			$this->appManager->expects($this->once())
				->method('enableApp')
				->with('enterprise_key');
			$this->logger->expects($this->once())
				->method('error');
		}

		$prevCLI = \OC::$CLI;
		\OC::$CLI = $isCli;

		include(__DIR__ . '/../../appinfo/app.php');

		\OC::$CLI = $prevCLI;
	}
}
