<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\AppInfo;


use OCA\Firewall\AppInfo\Application;
use Test\TestCase;

class ApplicationTest extends TestCase {
	/** @var Application */
	protected $application;

	protected function setUp() {
		parent::setUp();

		$this->application = new Application();
	}

	public function dataContainerQuery() {
		return [
			['OCA\Firewall\Config', '\OCA\Firewall\Config'],
			['OCA\Firewall\Context', '\OCA\Firewall\Context'],
			['OCA\Firewall\Logger', '\OCA\Firewall\Logger'],
			['OCA\Firewall\RuleFactory', '\OCA\Firewall\RuleFactory'],
			['OCA\Firewall\Ruler', '\OCA\Firewall\Ruler'],
			['OCA\Firewall\Rules', '\OCA\Firewall\Rules'],
			['OCA\Firewall\Firewall', '\OCA\Firewall\Firewall'],
		];
	}

	/**
	 * @dataProvider dataContainerQuery
	 *
	 * @param string $query
	 * @param string $class
	 */
	public function testContainerQuery($query, $class) {
		$object = $this->application->getContainer()->query($query);
		$this->assertInstanceOf($class, $object);
	}

	public function testBoot() {
		/** @var \OCA\Firewall\AppInfo\Application|\PHPUnit_Framework_MockObject_MockObject $application */
		$application = $this->getMockBuilder('OCA\Firewall\AppInfo\Application')
			->disableOriginalConstructor()
			->setMethods([
				'getContainer',
			])
			->getMock();

		$firewall = $this->getMockBuilder('OCA\Firewall\Firewall')
			->disableOriginalConstructor()
			->getMock();
		$firewall->expects($this->once())
			->method('boot');

		$container = $this->getMockBuilder('OCP\AppFramework\IAppContainer')
			->disableOriginalConstructor()
			->getMock();
		$container->expects($this->once())
			->method('query')
			->with('OCA\Firewall\Firewall')
			->willReturn($firewall);

		$application->expects($this->once())
			->method('getContainer')
			->willReturn($container);

		$application->boot();
	}
}
