<?php
/**
 * @author Clark Tomlinson  <clark@owncloud.com>
 * @since 2/3/15, 1:30 PM
 * @copyright 2015 ownCloud, Inc.
 *
 * This file is licensed under the Affero General Public License version 3 or later.
 * See the COPYING-README file.
 */

namespace OCA\Firewall\Tests\Helpers;


use OCA\Firewall\Ruler;
use OCA\Firewall\Rules;
use Test\TestCase;

abstract class RuleTester extends TestCase {

	/** @var \OCA\Firewall\Ruler */
	protected $ruler;

	/** @var \OCA\Firewall\Rules\Contracts\Rule */
	protected $rule;

	/** @var \OCA\Firewall\Context|\PHPUnit_Framework_MockObject_MockObject */
	protected $context;

	public function setUp() {
		parent::setUp();

		$this->ruler = new Ruler();

		$this->context = $this->getMockBuilder('OCA\Firewall\Context')
			->disableOriginalConstructor()
			->getMock();

		$class = $this->getRuleClass();
		if ($class !== null) {
			$this->rule = new $class(
				$this->ruler,
				$this->context
			);
		}
	}

	/**
	 * @return string|null
	 */
	abstract protected function getRuleClass();

	/**
	 * @return array
	 */
	abstract public function dataValidateRuleValue();

	/**
	 * @dataProvider dataValidateRuleValue
	 *
	 * @param mixed $value
	 * @param bool $throws
	 */
	public function testValidateRuleValue($value, $throws) {
		try {
			$this->rule->validateRuleValue($value, 'test');
			$this->assertFalse($throws);
		} catch (\OutOfBoundsException $e) {
			$this->assertTrue($throws, $e->getMessage());
		}
	}

	public function testGetValidValues() {
		$value = $this->rule->getValidValues();

		if (!is_array($value) && $value !== null) {
			$this->fail('Asserting that getValidOperators returns an array or null');
		}
	}

	public function testGetValidOperators() {
		$validOperators = $this->invokePrivate($this->rule, 'getValidOperators');
		$this->assertInternalType('array', $validOperators, 'Asserting that getValidOperators returns an array');
		$this->assertNotEmpty($validOperators, 'Asserting that getValidOperators does not return an empty array');

		/** @var \Hoa\Ruler\Visitor\Asserter $asserter */
		$asserter = $this->ruler->getAsserter();
		foreach ($validOperators as $operator) {
			if ($operator !== 'between') {
				$this->assertNotNull($asserter->getOperator($operator), 'Asserting that the operator exists');
			} else {
				$this->assertSame('\OCA\Firewall\Rules\Time', $this->getRuleClass(), 'Asserting that between operator is only allowed for Time');
			}
		}
	}
}
