<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\Config;
use Test\TestCase;

class ConfigTest extends TestCase {

	/** @var \OCP\IConfig|\PHPUnit_Framework_MockObject_MockObject */
	protected $ocConfig;

	/** @var \OCA\Firewall\Config */
	protected $config;


	protected function setUp() {
		parent::setUp();

		$this->ocConfig = $this->getMockBuilder('OCP\IConfig')
			->disableOriginalConstructor()
			->getMock();

		$this->config =  new Config(
			$this->ocConfig
		);
	}

	public function dataGetRules() {
		return [
			[json_encode(['rule1', 'rule2']), true, ['rule1', 'rule2']],
			[json_encode(['rule1', 'rule2']), false, json_encode(['rule1', 'rule2'])],
			[json_encode([]), true, []],
			[json_encode([]), false, json_encode([])],
			['[{}]', true, []],
			['[{}]', false, '{}'],
		];
	}

	/**
	 * @dataProvider dataGetRules
	 *
	 * @param string $config
	 * @param bool $decode
	 * @param mixed $expected
	 */
	public function testGetRules($config, $decode, $expected) {
		$this->ocConfig->expects($this->once())
			->method('getSystemValue')
			->with('firewall.rules', '{}')
			->willReturn($config);

		$this->assertSame($expected, $this->config->getRules($decode));
	}

	public function dataSetRules() {
		return [
			[[]],
			[json_encode([])],
			['foo'],
		];
	}

	/**
	 * @dataProvider dataSetRules
	 *
	 * @param mixed $input
	 */
	public function testSetRules($input) {
		$this->ocConfig->expects($this->once())
			->method('setSystemValue')
			->with('firewall.rules', $input);

		$this->config->setRules($input);
	}

	public function dataSetDebugLevel() {
		return [
			[1, 1],
			[2, 2],
			['1', 1],
			['foo', 0],
		];
	}

	/**
	 * @dataProvider dataSetDebugLevel
	 *
	 * @param mixed $input
	 * @param int $expected
	 */
	public function testSetDebugLevel($input, $expected) {
		$this->ocConfig->expects($this->once())
			->method('setSystemValue')
			->with('firewall.debug', $expected);

		$this->config->setDebug($input);
	}

	public function dataGetDebugLevel() {
		return [
			[1, 1],
			[2, 2],
			['1', 1],
			['foo', 0],
		];
	}

	/**
	 * @dataProvider dataGetDebugLevel
	 *
	 * @param mixed $config
	 * @param int $expected
	 */
	public function testGetDebugLevel($config, $expected) {
		$this->ocConfig->expects($this->once())
			->method('getSystemValue')
			->with('firewall.debug', 1)
			->willReturn($config);

		$this->assertSame($expected, $this->config->getDebugLevel());
	}

	public function dataGetBrandedClients() {
		return [
			[0, []],
			[1, []],
			['', []],
			['1', []],
			['foo', []],
			[[], []],
			[['test' => 'android'], ['test' => 'android']],
		];
	}

	/**
	 * @dataProvider dataGetBrandedClients
	 *
	 * @param mixed $config
	 * @param int $expected
	 */
	public function testGetBrandedClients($config, $expected) {
		$this->ocConfig->expects($this->once())
			->method('getSystemValue')
			->with('firewall.branded_clients', [])
			->willReturn($config);

		$this->assertSame($expected, $this->config->getBrandedClients());
	}
}
