<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\Logger;
use Test\TestCase;

class LoggerTest extends TestCase {

	/** @var \OCP\ILogger|\PHPUnit_Framework_MockObject_MockObject */
	private $logger;


	protected function setUp() {
		parent::setUp();

		$this->logger = $this->getMockBuilder('OCP\ILogger')
			->disableOriginalConstructor()
			->getMock();
	}

	public function dataOnSkip() {
		return [
			[Logger::LOGGING_OFF, false, false],
			[Logger::LOGGING_OFF, true, false],
			[Logger::LOGGING_FAILURES, false, false],
			[Logger::LOGGING_FAILURES, true, false],
			[Logger::LOGGING_ON, false, true],
			[Logger::LOGGING_ON, true, true],
		];
	}

	/**
	 * @dataProvider dataOnSkip
	 *
	 * @param int $logLevel
	 * @param bool $isAdmin
	 * @param bool $expected
	 */
	public function testOnSkip($logLevel, $isAdmin, $expected) {
		$logger = new Logger(
			$this->logger,
			$logLevel
		);

		$this->logger->expects($expected ? $this->once() : $this->never())
			->method('debug')
			->with(
				$isAdmin ? 'Firewall rule skipped for admin' : 'Firewall rule skipped for not logged in user',
				['app' => 'firewall']
			);

		$logger->onSkip($isAdmin);
	}

	public function dataOnPass() {
		return [
			[Logger::LOGGING_OFF, false],
			[Logger::LOGGING_FAILURES, false],
			[Logger::LOGGING_ON, true],
		];
	}

	/**
	 * @dataProvider dataOnPass
	 *
	 * @param int $logLevel
	 * @param bool $expected
	 */
	public function testOnPass($logLevel, $expected) {
		$logger = new Logger(
			$this->logger,
			$logLevel
		);

		$this->logger->expects($expected ? $this->once() : $this->never())
			->method('debug')
			->with('Firewall rule passed', ['app' => 'firewall']);

		$logger->onPass();
	}

	public function dataOnFail() {
		return [
			[Logger::LOGGING_OFF, 'offender1', false],
			[Logger::LOGGING_FAILURES, 'offender2', true],
			[Logger::LOGGING_ON, 'offender3', true],
		];
	}

	/**
	 * @dataProvider dataOnFail
	 *
	 * @param int $logLevel
	 * @param string $offendingRule
	 * @param bool $expected
	 */
	public function testGetRequestType($logLevel, $offendingRule, $expected) {
		$logger = new Logger(
			$this->logger,
			$logLevel
		);

		$this->logger->expects($expected ? $this->once() : $this->never())
			->method('debug')
			->with('Firewall validation on rule "{offender}"', [
				'app' => 'firewall',
				'offender' => $offendingRule,
			]);

		$logger->onFail($offendingRule);
	}
}
