<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Lib;


use OCA\Firewall\AppInfo\Application;
use OCA\Firewall\RuleFactory;
use Test\TestCase;

class RuleFactoryTest extends TestCase {

	/** @var RuleFactory */
	protected $factory;

	protected function setUp() {
		parent::setUp();

		$app = new Application();
		$this->factory = new RuleFactory($app->getContainer());
	}

	public function dataGetRuleInstance() {
		return [
			['cidr', 'OCA\Firewall\Rules\CIDR'],
			['cidr6', 'OCA\Firewall\Rules\CIDR'],
			['deviceType', 'OCA\Firewall\Rules\DeviceType'],
			['regex', 'OCA\Firewall\Rules\Regex'],
			['request', 'OCA\Firewall\Rules\Request'],
			['request-url', 'OCA\Firewall\Rules\RequestUrl'],
			['filetype', 'OCA\Firewall\Rules\FileType'],
			['sizeup', 'OCA\Firewall\Rules\FileSize'],
			['subnet', 'OCA\Firewall\Rules\SubNet'],
			['subnet6', 'OCA\Firewall\Rules\SubNet'],
			['time', 'OCA\Firewall\Rules\Time'],
			['userAgent', 'OCA\Firewall\Rules\UserAgent'],
			['userGroup', 'OCA\Firewall\Rules\UserGroup'],
		];
	}

	/**
	 * @dataProvider dataGetRuleInstance
	 *
	 * @param string $ruleId
	 * @param string $expectedClass
	 */
	public function testGetRuleInstance($ruleId, $expectedClass) {
		$service = $this->factory->getRuleInstance($ruleId);
		$this->assertInstanceOf($expectedClass, $service);
	}

	/**
	 * @expectedException \InvalidArgumentException
	 * @expectedMessage Unknown firewall rule id "Does Not Exist"
	 */
	public function testGetRuleInstanceInvalid() {
		$this->factory->getRuleInstance('Does Not Exist');
	}

	public function testGetRules() {
		$rules = [
			'cidr',
			'cidr6',
			'deviceType',
			'regex',
			'request',
			'request-url',
			'filetype',
			'sizeup',
			'subnet',
			'subnet6',
			'time',
			'userAgent',
			'userGroup',
		];

		$actual = $this->factory->getRules();
		$this->assertInternalType('array', $actual);
		$this->assertCount(sizeof($rules), $actual);

		foreach ($rules as $rule) {
			$this->assertTrue(in_array($rule, $actual), 'Asserting that rule is returned');
		}
	}
}
