<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class FileSizeTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			['test', true],
			['http://www.owncloud.dev/index.php/apps/files/', true],
			[0, false],
			[42, false],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\FileSize';
	}

	const SIZE = 10485760;

	public function dataDoCheck() {
		return [
			// No size, always doesn't match
			[Ruler::OPERATOR_LESS, 0, false],
			[Ruler::OPERATOR_LESS_OR_EQUALS, 0, false],
			[Ruler::OPERATOR_GREATER, 0, false],
			[Ruler::OPERATOR_GREATER_OR_EQUALS, 0, false],

			[Ruler::OPERATOR_LESS, self::SIZE - 1, true],
			[Ruler::OPERATOR_LESS, self::SIZE, false],
			[Ruler::OPERATOR_LESS, self::SIZE + 1, false],

			[Ruler::OPERATOR_LESS_OR_EQUALS, self::SIZE - 1, true],
			[Ruler::OPERATOR_LESS_OR_EQUALS, self::SIZE, true],
			[Ruler::OPERATOR_LESS_OR_EQUALS, self::SIZE + 1, false],

			[Ruler::OPERATOR_GREATER, self::SIZE - 1, false],
			[Ruler::OPERATOR_GREATER, self::SIZE, false],
			[Ruler::OPERATOR_GREATER, self::SIZE + 1, true],

			[Ruler::OPERATOR_GREATER_OR_EQUALS, self::SIZE - 1, false],
			[Ruler::OPERATOR_GREATER_OR_EQUALS, self::SIZE, true],
			[Ruler::OPERATOR_GREATER_OR_EQUALS, self::SIZE + 1, true],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $operator
	 * @param int $size
	 * @param bool $expected
	 */
	public function testDoCheck($operator, $size, $expected) {
		$this->context->expects($this->atLeastOnce())
			->method('getUploadSize')
			->willReturn($size);

		$this->assertSame($expected, $this->rule->doCheck($operator, self::SIZE));
	}
}
