<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class FileTypeTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			['test', false],
			['text/plain', false],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\FileType';
	}

	public function dataDoCheck() {
		return [
			// No file always does not match
			[Ruler::OPERATOR_EQUALS, null, 'text/plain', false],
			[Ruler::OPERATOR_NOT_EQUALS, null, 'text/plain', false],
			[Ruler::OPERATOR_BEGINS_WITH, null, 'text/plain', false],
			[Ruler::OPERATOR_NOT_BEGINS_WITH, null, 'text/plain', false],
			[Ruler::OPERATOR_ENDS_WITH, null, 'text/plain', false],
			[Ruler::OPERATOR_NOT_ENDS_WITH, null, 'text/plain', false],


			[Ruler::OPERATOR_EQUALS, 'text/plain', 'text/plain', true],
			[Ruler::OPERATOR_EQUALS, 'text/plain', 'audio/mpeg', false],
			[Ruler::OPERATOR_NOT_EQUALS, 'text/plain', 'text/plain', false],
			[Ruler::OPERATOR_NOT_EQUALS, 'text/plain', 'audio/mpeg', true],

			[Ruler::OPERATOR_BEGINS_WITH, 'text/plain', 'text/', true],
			[Ruler::OPERATOR_BEGINS_WITH, 'text/plain', 'audio/', false],
			[Ruler::OPERATOR_NOT_BEGINS_WITH, 'text/plain', 'text/', false],
			[Ruler::OPERATOR_NOT_BEGINS_WITH, 'text/plain', 'audio/', true],

			[Ruler::OPERATOR_ENDS_WITH, 'text/plain', '/plain', true],
			[Ruler::OPERATOR_ENDS_WITH, 'text/plain', '/mpeg', false],
			[Ruler::OPERATOR_NOT_ENDS_WITH, 'text/plain', '/plain', false],
			[Ruler::OPERATOR_NOT_ENDS_WITH, 'text/plain', '/mpeg', true],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $operator
	 * @param string $upload
	 * @param string $ruleValue
	 * @param bool $expected
	 */
	public function testDoCheck($operator, $upload, $ruleValue, $expected) {
		$this->context->expects($this->atLeastOnce())
			->method('getUploadType')
			->willReturn($upload);

		$this->assertSame($expected, $this->rule->doCheck($operator, $ruleValue));
	}
}
