<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class RequestUrlTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			['test', false],
			['http://www.owncloud.dev/index.php/apps/files/', false],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\RequestUrl';
	}

	public function dataDoCheck() {
		return [
			['http://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_EQUALS, true],
			['https://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_EQUALS, false],
			['http://www.owncloud.dev/index.php/apps/files/', Ruler::OPERATOR_EQUALS, false],
			['http://www.owncloud.dev/index.php', Ruler::OPERATOR_EQUALS, false],

			['http://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_BEGINS_WITH, true],
			['https://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_BEGINS_WITH, false],
			['http://www.owncloud.dev/index.php/apps/files/', Ruler::OPERATOR_BEGINS_WITH, true],
			['http://www.owncloud.dev/index.php', Ruler::OPERATOR_BEGINS_WITH, false],
			['https://http://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_BEGINS_WITH, false],
			['www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_BEGINS_WITH, false],

			['http://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_ENDS_WITH, true],
			['https://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_ENDS_WITH, false],
			['http://www.owncloud.dev/index.php/apps/files/', Ruler::OPERATOR_ENDS_WITH, false],
			['http://www.owncloud.dev/index.php', Ruler::OPERATOR_ENDS_WITH, false],
			['https://http://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_ENDS_WITH, true],
			['www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_ENDS_WITH, false],

			['http://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_CONTAINS, true],
			['ttp://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_CONTAINS, false],
			['http://www.owncloud.dev/index.php/apps/file', Ruler::OPERATOR_CONTAINS, false],
			['http://www.owncloud.dev/index.php/apps/files/', Ruler::OPERATOR_CONTAINS, true],
			['ahttp://www.owncloud.dev/index.php/apps/files', Ruler::OPERATOR_CONTAINS, true],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $requestUrl
	 * @param string $operator
	 * @param bool $expected
	 */
	public function testDoCheck($requestUrl, $operator, $expected) {
		$this->context->expects($this->once())
			->method('getRequestFullUrl')
			->willReturn($requestUrl);

		$this->assertSame($expected, $this->rule->doCheck($operator, 'http://www.owncloud.dev/index.php/apps/files'));
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param string $requestUrl
	 * @param string $operator
	 * @param bool $expected
	 */
	public function testDoCheckNotMatch($requestUrl, $operator, $expected) {
		$operator = 'not_' . $operator;

		$this->context->expects($this->once())
			->method('getRequestFullUrl')
			->willReturn($requestUrl);

		$this->assertSame(!$expected, $this->rule->doCheck($operator, 'http://www.owncloud.dev/index.php/apps/files'));
	}
}
