<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Tests\Helpers\RuleTester;

class UserAgentTest extends RuleTester {

	public function dataValidateRuleValue() {
		return [
			['Firefox/37.0', false],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return '\OCA\Firewall\Rules\UserAgent';
	}

	public function dataDoCheck() {
		return [
			['Firefox/37.0', 'Firefox/37.0', true],
			['Firefox/38.0', 'Firefox/37.0', false],
		];
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param $userAgent
	 * @param $ruleAgent
	 * @param $expected
	 */
	public function testDoCheckEqual($userAgent, $ruleAgent, $expected) {
		$this->context->expects($this->once())
			->method('getUserAgent')
			->willReturn($userAgent);

		$this->assertSame($expected, $this->rule->doCheck(Ruler::OPERATOR_EQUALS, $ruleAgent));
	}

	/**
	 * @dataProvider dataDoCheck
	 *
	 * @param $userAgent
	 * @param $ruleAgent
	 * @param $expected
	 */
	public function testDoCheckNotEqual($userAgent, $ruleAgent, $expected) {
		$this->context->expects($this->once())
			->method('getUserAgent')
			->willReturn($userAgent);

		$this->assertSame(!$expected, $this->rule->doCheck(Ruler::OPERATOR_NOT_EQUALS, $ruleAgent));
	}

}
