<?php
/**
 * ownCloud Firewall
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Firewall\Tests\Rules;


use OCA\Firewall\Ruler;
use OCA\Firewall\Rules\UserGroup;
use OCA\Firewall\Tests\Helpers\RuleTester;

class UserGroupTest extends RuleTester {

	/** @var \OCP\IGroupManager|\PHPUnit_Framework_MockObject_MockObject */
	protected $groupManager;

	public function setUp() {
		parent::setUp();

		$this->groupManager = $this->getMockBuilder('OCP\IGroupManager')
			->disableOriginalConstructor()
			->getMock();

		$this->rule = new UserGroup(
			$this->ruler,
			$this->context,
			$this->groupManager
		);
	}

	public function dataValidateRuleValue() {
		return [
			['test', false],
			[0, true],
			[42, true],
			[[], true],
			[['test'], true],
			[null, true],
		];
	}

	/**
	 * @return string
	 */
	protected function getRuleClass() {
		return null;
	}

	public function testGetValidValues() {
		$this->groupManager->expects($this->once())
			->method('search')
			->with('', null, null)
			->willReturn([]);

		parent::testGetValidValues();
	}

	/**
	 * @param string $gid
	 * @return \OCP\IGroup|\PHPUnit_Framework_MockObject_MockObject
	 */
	protected function getGroupMock($gid) {
		$group = $this->getMockBuilder('OCP\IGroup')
			->disableOriginalConstructor()
			->getMock();
		$group->expects($this->any())
			->method('getGID')
			->willReturn($gid);
		return $group;
	}

	public function dataGetValidValuesGroups() {
		return [
			[[], []],
			[[$this->getGroupMock('test'), $this->getGroupMock('admin')], ['admin' => 'admin', 'test' => 'test']],
			[[$this->getGroupMock('admin'), $this->getGroupMock('test')], ['admin' => 'admin', 'test' => 'test']],
		];
	}

	/**
	 * @dataProvider dataGetValidValuesGroups
	 *
	 * @param array $groups
	 * @param array $expected
	 */
	public function testGetValidValuesGroups(array $groups, array $expected) {
		$this->groupManager->expects($this->once())
			->method('search')
			->with('', null, null)
			->willReturn($groups);

		$this->assertSame($expected, $this->rule->getValidValues(), 'Asserting that getValidValues returns an array');

		// Grab a second time, no new request to the group manager should be done
		$this->assertSame($expected, $this->rule->getValidValues(), 'Asserting that getValidValues returns an array');
	}

	public function dataInGroup() {
		return [
			// Check if user is in group
			[
				// Groups
				[],
				// Operator
				Ruler::OPERATOR_EQUALS,
				// Expected
				false,
			],
			[
				['admin', 'foo', 'bar', 'baz'],
				Ruler::OPERATOR_EQUALS,
				false,
			],
			[
				['admin', 'foo', 'bar', 'test_group'],
				Ruler::OPERATOR_EQUALS,
				true,
			],
			[
				['test_group', 'foo', 'bar', 'test_group'],
				Ruler::OPERATOR_EQUALS,
				true,
			],
			[
				['test_group'],
				Ruler::OPERATOR_EQUALS,
				true,
			],

			// Check if user is not in group
			[
				[],
				Ruler::OPERATOR_NOT_EQUALS,
				true,
			],
			[
				['admin', 'foo', 'bar', 'baz'],
				Ruler::OPERATOR_NOT_EQUALS,
				true,
			],
			[
				['admin', 'foo', 'bar', 'test_group'],
				Ruler::OPERATOR_NOT_EQUALS,
				false,
			],
			[
				['test_group', 'foo', 'bar', 'test_group'],
				Ruler::OPERATOR_NOT_EQUALS,
				false,
			],
			[
				['test_group'],
				Ruler::OPERATOR_NOT_EQUALS,
				false,
			],
		];
	}

	/**
	 * @dataProvider dataInGroup
	 *
	 * @param array $groups
	 * @param string $operator
	 * @param bool $expected
	 */
	public function testInGroup(array $groups, $operator, $expected) {
		$this->context->expects($this->once())
			->method('getUserGroups')
			->willReturn($groups);

		$this->assertSame($expected, $this->rule->doCheck($operator, 'test_group'));
	}
}
