<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Db;

class ShibbolethUserMapper {

	/**
	 * FIXME make non-static
	 * Sets non-shib webDAV password
	 * @param string $uid user id
	 * @param string $password new webdav password
	 * @returns bool whether setting password succeeded
	 * @throws \OC\DatabaseException
	 */
	public static function setWebDAVPassword($uid, $password) {
		if (self::checkUserExists($uid)) {
			$hash = \OC::$server->getHasher()->hash($password);
			$query = \OC_DB::prepare('UPDATE `*PREFIX*shibboleth_user` SET `webdav_password` = ? WHERE LOWER(`eppn`) LIKE LOWER(?)');
			$result = $query->execute(array($hash, $uid));

			return $result ? true : false;
		}

		return false;
	}

	/**
	 * FIXME make non-static
	 * Clears non-shib webDAV password for given user
	 * @param string $uid user id
	 * @returns bool whether removing succeeded
	 * @throws \OC\DatabaseException
	 */
	public static function clearWebDAVPassword($uid) {
		if (self::checkUserExists($uid)) {
			$query = \OC_DB::prepare('UPDATE `*PREFIX*shibboleth_user` SET `webdav_password` = NULL WHERE LOWER(`eppn`) LIKE LOWER(?)');
			$result = $query->execute(array($uid));

			return $result ? true : false;
		}

		return false;
	}

	/**
	 * FIXME make non-static
	 * Checks if given user has non-shib webDAV password set
	 * @param string $uid user id
	 * @returns bool whether the password is set
	 * @throws \OC\DatabaseException
	 */
	public static function checkIfUserHasWebDAVPassword($uid) {
		if (self::checkUserExists($uid)) {
			$query = \OC_DB::prepare('SELECT `webdav_password` FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) LIKE LOWER(?)');
			$result = $query->execute(array($uid));
			$row = $result->fetchRow();
			if (!$row || empty($row['webdav_password'])) {
				return false;
			}
			return true;
		}

		return false;
	}

	/**
	 * FIXME make non-static
	 * check if a user exists
	 * @param string $uid the username
	 * @return boolean
	 * @throws \OC\DatabaseException
	 */
	public static function checkUserExists($uid) {
		$query = \OC_DB::prepare( 'SELECT * FROM `*PREFIX*shibboleth_user` WHERE LOWER(`eppn`) = LOWER(?)' );
		$result = $query->execute( array( $uid ));
		if ($result->fetchRow()) {
			return true;
		}

		return false;
	}
}
