<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Mode;


use OCP\IUserBackend;
use OCP\UserInterface;

interface Mode extends UserInterface, IUserBackend {

	/**
	 * Called after the shibboleth UserBackend has checked the environment
	 * allows the mode implementation to do further checks
	 * @param $uid string uid - eg for shibboleth session the eppn - for AD / ADFS the sAMAccountName
	 * @return bool
	 */
	public function isSessionActive($uid);

	/**
	 * returns a counts of the users
	 *
	 * @return int|bool
	 */
	public function countUsers();

	/**
	 * Check if the password is correct
	 * @param string $uid The username
	 * @param string $password The password
	 * @return string|false
	 *
	 * Check if the password is correct without logging in the user
	 * returns the user id or false
	 */
	public function checkPassword($uid, $password);

}