<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Command;

use OCP\IConfig;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Test\TestCase;

class MappingTest extends TestCase {

	/**
	 * @var Mapping
	 */
	private $command;

	/**
	 * @var IConfig
	 */
	private $config;

	/**
	 * @var InputInterface
	 */
	private $input;

	/**
	 * @var OutputInterface
	 */
	private $output;


	protected function setUp() {
		parent::setUp();

		$this->config = $this->getMockBuilder(
			'\OCP\IConfig')
			->disableOriginalConstructor()
			->getMock();
		$this->input = $this->getMockBuilder('Symfony\Component\Console\Input\InputInterface')
			->disableOriginalConstructor()
			->getMock();
		$this->output = $this->getMockBuilder('Symfony\Component\Console\Output\OutputInterface')
			->disableOriginalConstructor()
			->getMock();

		$this->command = new Mapping($this->config);
	}

	/**
	 * Test setting the mapping
	 * @dataProvider setProvider
	 */
	public function testSetMapping($shibSession, $uid, $email, $displayName) {
		$this->input->expects($this->at(0))
			->method('getOption')
			->with('shib-session')
			->will($this->returnValue($shibSession));
		$this->input->expects($this->at(1))
			->method('getOption')
			->with('uid')
			->will($this->returnValue($uid));
		$this->input->expects($this->at(2))
			->method('getOption')
			->with('email')
			->will($this->returnValue($email));
		$this->input->expects($this->at(3))
			->method('getOption')
			->with('display-name')
			->will($this->returnValue($displayName));

		$optionCount = count($shibSession) + count($uid) + count($email) + count($displayName);
		$currentOption = 0;
		if ($shibSession) {
			$this->config->expects($this->at($currentOption))
				->method('setAppValue')
				->with('user_shibboleth', 'env-source-shib-session', $shibSession);
			$currentOption++;
		}
		if ($uid) {
			$this->config->expects($this->at($currentOption))
				->method('setAppValue')
				->with('user_shibboleth', 'env-source-uid', $uid);
			$currentOption++;
		}
		if ($email) {
			$this->config->expects($this->at($currentOption))
				->method('setAppValue')
				->with('user_shibboleth', 'env-source-email', $email);
			$currentOption++;
		}
		if ($displayName) {
			$this->config->expects($this->at($currentOption))
				->method('setAppValue')
				->with('user_shibboleth', 'env-source-displayname', $displayName);
		}

		$this->config->expects($this->at($optionCount))
			->method('getAppValue')
			->with('user_shibboleth', 'env-source-shib-session')
			->will($this->returnValue($shibSession));
		$this->config->expects($this->at($optionCount + 1))
			->method('getAppValue')
			->with('user_shibboleth', 'env-source-uid')
			->will($this->returnValue($uid));
		$this->config->expects($this->at($optionCount + 2))
			->method('getAppValue')
			->with('user_shibboleth', 'env-source-email')
			->will($this->returnValue($email));
		$this->config->expects($this->at($optionCount + 3))
			->method('getAppValue')
			->with('user_shibboleth', 'env-source-displayname')
			->will($this->returnValue($displayName));

		$this->output
			->expects($this->at(0))
			->method('writeln')
			->with(
				'Checking "' . $shibSession . '" in environment to detect shibboleth session'
			);
		$this->output
			->expects($this->at(1))
			->method('writeln')
			->with(
				'Reading uid from "'.$uid.'"'
			);
		$this->output
			->expects($this->at(2))
			->method('writeln')
			->with(
				'Reading email from "'.$email.'"'
			);
		$this->output
			->expects($this->at(3))
			->method('writeln')
			->with(
				'Reading display name from "'.$displayName.'"'
			);

		$this->command->execute($this->input, $this->output);

	}

	public function setProvider () {
		return [
			[null, null, null, null],
			[null, null, null, 'TestDisplayName'],
			[null, null, 'TestEmail', null],
			[null, null, 'TestEmail', 'TestDisplayName'],
			[null, 'TestUid', null, null],
			[null, 'TestUid', null, 'TestDisplayName'],
			[null, 'TestUid', 'TestEmail', null],
			[null, 'TestUid', 'TestEmail', 'TestDisplayName'],
			['TestSession', null, null, null],
			['TestSession', null, null, 'TestDisplayName'],
			['TestSession', null, 'TestEmail', null],
			['TestSession', null, 'TestEmail', 'TestDisplayName'],
			['TestSession', 'TestUid', null, null],
			['TestSession', 'TestUid', null, 'TestDisplayName'],
			['TestSession', 'TestUid', 'TestEmail', null],
			['TestSession', 'TestUid', 'TestEmail', 'TestDisplayName'],
		];
	}

}