<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Command;

use OCA\User_Shibboleth\UserBackendFactory;
use OCP\IConfig;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Test\TestCase;

class ModeTest extends TestCase {

	/**
	 * @var Mode
	 */
	private $command;

	/**
	 * @var IConfig
	 */
	private $config;

	/**
	 * @var InputInterface
	 */
	private $input;

	/**
	 * @var OutputInterface
	 */
	private $output;


	protected function setUp() {
		parent::setUp();

		$this->config = $this->getMockBuilder(
			'\OCP\IConfig')
			->disableOriginalConstructor()
			->getMock();
		$this->input = $this->getMockBuilder('Symfony\Component\Console\Input\InputInterface')
			->disableOriginalConstructor()
			->getMock();
		$this->output = $this->getMockBuilder('Symfony\Component\Console\Output\OutputInterface')
			->disableOriginalConstructor()
			->getMock();

		$this->config->expects($this->once())
			->method('getAppValue')
			->with('user_shibboleth', 'mode', '')
			->will($this->returnValue('something'));

		$this->command = new Mode($this->config);
	}

	/**
	 * Test getting the mode
	 */
	public function testGetMode() {
		$this->input->expects($this->once())
			->method('getArgument')
			->with('mode'); // returns null by default

		$this->output
			->expects($this->once())
			->method('writeln')
			->with("User Shibboleth mode is 'something'");

		$this->command->execute($this->input, $this->output);

	}

	/**
	 * Test setting the mode
	 * @dataProvider setProvider
	 * @param $mode string
	 */
	public function testSetMode($mode) {

		$this->input->expects($this->once())
			->method('getArgument')
			->with('mode')
			->will($this->returnValue($mode));

		$this->output
			->expects($this->once())
			->method('writeln')
			->with("User Shibboleth mode changed from 'something' to '$mode'");

		$this->command->execute($this->input, $this->output);

	}
	/**
	 * Test setting the mode
	 */
	public function testSetInvalidMode() {

		$this->input->expects($this->once())
			->method('getArgument')
			->with('mode')
			->will($this->returnValue('invalidmode'));

		$this->output
			->expects($this->once())
			->method('writeln')
			->with($this->stringContains("Unknown mode 'invalidmode'"));

		$this->command->execute($this->input, $this->output);

	}

	public function setProvider () {
		return [
			[UserBackendFactory::MODE_NOT_ACTIVE],
			[UserBackendFactory::MODE_SSO_ONLY],
			[UserBackendFactory::MODE_AUTOPROVISION]
		];
	}

}