<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@butonic.de>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Controller;

use OCA\User_Shibboleth\UserBackendFactory;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\IConfig;
use OCP\IRequest;
use Test\TestCase;

class AdminControllerTest extends TestCase {

	/** @var IRequest */
	private $request;

	/** @var IConfig */
	private $config;

	/** @var AdminController */
	private $controller;

	public function setUp() {
		parent::setUp();

		$this->request = $this->getMock('\OCP\IRequest');
		$this->config = $this->getMock('\OCP\IConfig');

		$this->controller = new AdminController(
			'user_shibboleth',
			$this->request,
			$this->config
		);

	}


	public function testGetMode() {
		$this->config->expects($this->once())
			->method('getAppValue')
			->with('user_shibboleth', 'mode', UserBackendFactory::MODE_NOT_ACTIVE)
			->will($this->returnValue('somemode'));

		$expected = new DataResponse('somemode');
		$this->assertequals($expected, $this->controller->getMode());
	}

	/**
	 * @dataProvider setModeProvider
	 * @param $mode string
	 * @param $expected DataResponse
	 */
	public function testSetMode($mode, $expected) {
		$this->config->expects($this->any())
			->method('setAppValue')
			->with('user_shibboleth', 'mode', $mode);

		$this->assertEquals($expected, $this->controller->setMode($mode));
	}

	public function setModeProvider() {
		return [
			[UserBackendFactory::MODE_NOT_ACTIVE, new DataResponse()],
			[UserBackendFactory::MODE_SSO_ONLY, new DataResponse()],
			[UserBackendFactory::MODE_AUTOPROVISION, new DataResponse()],
			['invalid', new DataResponse(
				['message' => 'Could not change mode'],
				Http::STATUS_UNPROCESSABLE_ENTITY)
			],
		];
	}
}