<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Mode;

use OCP\ILogger;
use OCP\IUserManager;
use Test\TestCase;

class SingleSignOnTest extends TestCase {

	/** @var ILogger */
	private $logger;

	/** @var IUserManager */
	private $userManager;

	/** @var SingleSignOn */
	private $mode;

	public function setUp() {
		parent::setUp();

		$this->logger = $this->getMockBuilder(
			'\OCP\ILogger')
			->disableOriginalConstructor()
			->getMock();
		$this->userManager = $this->getMockBuilder(
			'\OCP\IUserManager')
			->disableOriginalConstructor()
			->getMock();

		$this->mode = new SingleSignOn($this->logger, $this->userManager);
	}

	public function testIsSessionActiveForExistingUser() {
		$this->userManager->expects($this->once())
			->method('userExists')
			->with('existing@uid')
			->will($this->returnValue(true));

		$this->assertTrue($this->mode->isSessionActive('existing@uid'));
	}

	public function testIsSessionActiveForNonExistingUser() {
		$this->userManager->expects($this->once())
			->method('userExists')
			->with('notexisting@uid')
			->will($this->returnValue(false));

		$this->assertFalse($this->mode->isSessionActive('notexisting@uid'));
	}

	/**
	 * @dataProvider actionsProvider
	 * @param $action int
	 */
	public function testImplementsActions($action) {
		$this->assertFalse($this->mode->implementsActions($action));
	}

	public function actionsProvider () {
		return [
			[\OC_User_Backend::CREATE_USER],
			[\OC_User_Backend::SET_PASSWORD],
			[\OC_User_Backend::CHECK_PASSWORD],
			[\OC_User_Backend::GET_HOME],
			[\OC_User_Backend::GET_DISPLAYNAME],
			[\OC_User_Backend::SET_DISPLAYNAME],
			[\OC_User_Backend::PROVIDE_AVATAR],
			[\OC_User_Backend::COUNT_USERS],
		];
	}

	public function testGetBackendName() {
		$this->assertSame('Shibboleth (SSO)', $this->mode->getBackendName());
	}
}