<?php
/**
 * ownCloud
 *
 * @author Thomas Müller <deepdiver@owncloud.com>
 * @author Iago López Galeiras
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth;

use OCA\User_Shibboleth\Db\ShibbolethUserMapper;
use Test\TestCase;

class ProvisioningTest extends TestCase {

	/** @var string */
	private $savedMode;

	/** @var string */
	private $savedEnvSourceShibbolethSession;

	/** @var string */
	private $savedEnvSourceUid;

	/**
	 * @var UserBackend
	 */
	private $backend;

	/**
	 * @var string
	 */
	private $currentUser;

	/** @var \OCP\IConfig */
	private $config;

	/** @var \OCP\ILogger */
	private $logger;

	/** @var \OCP\IUserManager */
	private $userManager;

	public function setUp() {
		parent::setUp();

		$this->logger = $this->getMockBuilder(
			'\OCP\ILogger')
			->disableOriginalConstructor()
			->getMock();
		$this->userManager = $this->getMockBuilder(
			'\OCP\IUserManager')
			->disableOriginalConstructor()
			->getMock();

		$this->config = \OC::$server->getConfig();

		$this->savedMode = $this->config->getAppValue(
			'user_shibboleth', 'mode', null
		);
		$this->config->setAppValue(
			'user_shibboleth', 'mode', UserBackendFactory::MODE_AUTOPROVISION
		);

		$this->savedEnvSourceShibbolethSession = $this->config->getAppValue(
			'user_shibboleth', 'env-source-shib-session', null
		);
		$this->config->setAppValue(
			'user_shibboleth', 'env-source-shib-session', 'Test-Shib-Session-ID'
		);

		$this->savedEnvSourceUid = $this->config->getAppValue(
			'user_shibboleth', 'env-source-uid', null
		);
		$this->config->setAppValue(
			'user_shibboleth', 'env-source-uid', 'test-eppn'
		);

		$this->currentUser = uniqid('shib-user-') . '@example.com';
		$_SERVER["PHP_AUTH_USER"] = $this->currentUser;
		$_SERVER['test-eppn'] = $this->currentUser;
		$_SERVER['Test-Shib-Session-ID'] = uniqid('shib');

		$factory = new UserBackendFactory(
			$this->config, $this->logger, $this->userManager
		);
		$this->backend = $factory->createBackend();
	}

	public function tearDown() {
		if ($this->savedEnvSourceShibbolethSession) {
			$this->config->setAppValue(
				'user_shibboleth',
				'env-source-shib-session',
				$this->savedEnvSourceShibbolethSession
			);
		} else {
			$this->config->deleteAppValue(
				'user_shibboleth', 'env-source-shib-session'
			);
		}
		if ($this->savedEnvSourceUid) {
			$this->config->setAppValue(
				'user_shibboleth', 'env-source-uid', $this->savedEnvSourceUid
			);
		} else {
			$this->config->deleteAppValue('user_shibboleth', 'env-source-uid');
		}
		if ($this->savedMode) {
			$this->config->setAppValue(
				'user_shibboleth', 'mode', $this->savedMode
			);
		} else {
			$this->config->deleteAppValue('user_shibboleth', 'mode');
		}
		parent::tearDown();
	}

	public function testDeleteCredential() {
		$result = $this->backend->isSessionActive();
		$this->assertTrue($result);

		ShibbolethUserMapper::setWebDAVPassword($this->currentUser, '1234');
		$this->assertTrue(ShibbolethUserMapper::checkIfUserHasWebDAVPassword($this->currentUser));

		$parameters = array('userid' => $this->currentUser);
		$return = Provisioning::deleteCredential($parameters);
		$this->assertTrue($return->succeeded());
		$this->assertFalse(ShibbolethUserMapper::checkIfUserHasWebDAVPassword($this->currentUser));

		$parameters = array('userid' => "bogusUser");
		$return = Provisioning::deleteCredential($parameters);
		$this->assertEquals(\OCP\API::RESPOND_NOT_FOUND, $return->getStatusCode());

		$parameters = array();
		$return = Provisioning::deleteCredential($parameters);
		$this->assertEquals(\OCP\API::RESPOND_SERVER_ERROR, $return->getStatusCode());
	}

}
