<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\User_Shibboleth\Tests;

use OCA\User_Shibboleth\UserBackendFactory;
use OCP\IConfig;
use OCP\ILogger;
use OCP\IUserManager;
use Test\TestCase;

class UserBackendFactoryTest extends TestCase {

	/**
	 * @var IConfig
	 */
	private $config;
	/**
	 * @var ILogger
	 */
	private $logger;
	/**
	 * @var IUserManager
	 */
	private $userManager;

	public function setUp() {
		parent::setUp();

		$this->logger = $this->getMockBuilder(
			'\OCP\ILogger')
			->disableOriginalConstructor()
			->getMock();
		$this->config = $this->getMockBuilder(
			'\OCP\IConfig')
			->disableOriginalConstructor()
			->getMock();
		$this->userManager = $this->getMockBuilder(
			'\OCP\IUserManager')
			->disableOriginalConstructor()
			->getMock();
	}

	public function testCheckAndUpdateLegacyConfig() {
		$this->config->expects($this->once())
			->method('getAppValue')
			->will($this->returnValue(1));
		$this->config->expects($this->once())
			->method('deleteAppValue')
			->with(
				$this->equalTo('user_shibboleth'),
				$this->equalTo('shibboleth_active')
			);
		$this->config->expects($this->once())
			->method('setAppValue')
			->with(
				$this->equalTo('user_shibboleth'),
				$this->equalTo('mode'),
				$this->equalTo(UserBackendFactory::MODE_AUTOPROVISION)
			);

		$factory = new UserBackendFactory($this->config ,$this->logger, $this->userManager);
		$factory->checkAndUpdateLegacyConfig(null);
	}


	/**
	 * @dataProvider createBackendDataProvider
	 * @param $mode string
	 * @param $modeClass string
	 * @throws \OCA\User_Shibboleth\ConfigurationError
	 */
	public function testCreateBackend($mode, $modeClass) {
		$this->config->expects($this->once())
			->method('getAppValue')
			->will($this->returnValue($mode));

		$factory = new UserBackendFactory($this->config ,$this->logger, $this->userManager);
		$backend = $factory->createBackend();

		$this->assertInstanceOf('OCA\User_Shibboleth\UserBackend', $backend);
		$this->assertInstanceOf($modeClass, $backend->getMode());
	}

	public function createBackendDataProvider () {
		return [
			[UserBackendFactory::MODE_SSO_ONLY, 'OCA\User_Shibboleth\Mode\SingleSignOn'],
			[UserBackendFactory::MODE_AUTOPROVISION, 'OCA\User_Shibboleth\Mode\AutoProvisioning'],
		];
	}

	/**
	 * @throws \OCA\User_Shibboleth\ConfigurationError
	 */
	public function testCreateNullBackend() {
		$this->config->expects($this->once())
			->method('getAppValue')
			->will($this->returnValue(UserBackendFactory::MODE_NOT_ACTIVE));

		$factory = new UserBackendFactory($this->config ,$this->logger, $this->userManager);
		$backend = $factory->createBackend();

		$this->assertNull($backend);
	}
	/**
	 * @expectedException \OCA\User_Shibboleth\ConfigurationError
	 */
	public function testCreateBackendConfigurationException() {
		$this->config->expects($this->once())
			->method('getAppValue')
			->will($this->returnValue('unknownmode'));

		$factory = new UserBackendFactory($this->config ,$this->logger, $this->userManager);
		$factory->createBackend();
	}

}
